
from typing import Any

from discord import Guild, Member, User
from discord.abc import Messageable
from redbot.core import commands
from redbot.core.bot import Red


#@type_registry.register(key="type")
class Type(object):
    key = "type" # this should ALWAYS be the same as the registry key, and should NEVER be localized
    string = "type"
    verb = "typed"
    embed_desc = "been"
    channel = False # if this is True, the overridden handler methods should be typed with `discord.abc.Messageable` instead of `discord.Member | discord.User`

    def __str__(self) -> str:
        return self.string

    @classmethod
    async def handler(cls, ctx: commands.Context, target: Member | User | Messageable, silent: bool, **kwargs) -> 'Type': # pylint: disable=unused-argument
        """This method should be overridden by any child classes, but should retain the same starting keyword arguments."""
        raise NotImplementedError

    @classmethod
    async def resolve_handler(cls, bot: Red, guild: Guild, target: Member | User | Messageable, reason: str | None = None, **kwargs) -> Any: # pylint: disable=unused-argument
        """This method should be overridden by any resolvable child classes, but should retain the same keyword arguments.
        If your moderation type should not be resolvable, do not override this."""
        raise NotImplementedError

    @classmethod
    async def expiry_handler(cls, bot: Red, guild: Guild, target: Member | User | Messageable, **kwargs) -> Any: # pylint: disable=unused-argument
        """This method should be overridden by any expirable child classes, but should retain the same keyword arguments.
        If your moderation type should not expire, do not override this."""
        raise NotImplementedError
